# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import records_pb2 as protocol_dot_records__pb2


class RecordsStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetCategoryList = channel.unary_unary(
                '/records.Records/GetCategoryList',
                request_serializer=protocol_dot_records__pb2.Empty.SerializeToString,
                response_deserializer=protocol_dot_records__pb2.CategoryList.FromString,
                )
        self.GetCategoryListForUser = channel.unary_unary(
                '/records.Records/GetCategoryListForUser',
                request_serializer=protocol_dot_records__pb2.User.SerializeToString,
                response_deserializer=protocol_dot_records__pb2.CategoryList.FromString,
                )
        self.GetRecordById = channel.unary_unary(
                '/records.Records/GetRecordById',
                request_serializer=protocol_dot_records__pb2.RecordRequest.SerializeToString,
                response_deserializer=protocol_dot_records__pb2.Record.FromString,
                )
        self.GetLastGroupForUser = channel.unary_unary(
                '/records.Records/GetLastGroupForUser',
                request_serializer=protocol_dot_records__pb2.User.SerializeToString,
                response_deserializer=protocol_dot_records__pb2.RecordList.FromString,
                )
        self.GetRecords = channel.unary_unary(
                '/records.Records/GetRecords',
                request_serializer=protocol_dot_records__pb2.RecordQuery.SerializeToString,
                response_deserializer=protocol_dot_records__pb2.RecordList.FromString,
                )
        self.CountRecords = channel.unary_unary(
                '/records.Records/CountRecords',
                request_serializer=protocol_dot_records__pb2.RecordQuery.SerializeToString,
                response_deserializer=protocol_dot_records__pb2.RecordList.FromString,
                )


class RecordsServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetCategoryList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCategoryListForUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRecordById(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetLastGroupForUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRecords(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CountRecords(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_RecordsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetCategoryList': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCategoryList,
                    request_deserializer=protocol_dot_records__pb2.Empty.FromString,
                    response_serializer=protocol_dot_records__pb2.CategoryList.SerializeToString,
            ),
            'GetCategoryListForUser': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCategoryListForUser,
                    request_deserializer=protocol_dot_records__pb2.User.FromString,
                    response_serializer=protocol_dot_records__pb2.CategoryList.SerializeToString,
            ),
            'GetRecordById': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRecordById,
                    request_deserializer=protocol_dot_records__pb2.RecordRequest.FromString,
                    response_serializer=protocol_dot_records__pb2.Record.SerializeToString,
            ),
            'GetLastGroupForUser': grpc.unary_unary_rpc_method_handler(
                    servicer.GetLastGroupForUser,
                    request_deserializer=protocol_dot_records__pb2.User.FromString,
                    response_serializer=protocol_dot_records__pb2.RecordList.SerializeToString,
            ),
            'GetRecords': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRecords,
                    request_deserializer=protocol_dot_records__pb2.RecordQuery.FromString,
                    response_serializer=protocol_dot_records__pb2.RecordList.SerializeToString,
            ),
            'CountRecords': grpc.unary_unary_rpc_method_handler(
                    servicer.CountRecords,
                    request_deserializer=protocol_dot_records__pb2.RecordQuery.FromString,
                    response_serializer=protocol_dot_records__pb2.RecordList.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'records.Records', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Records(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetCategoryList(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/records.Records/GetCategoryList',
            protocol_dot_records__pb2.Empty.SerializeToString,
            protocol_dot_records__pb2.CategoryList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCategoryListForUser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/records.Records/GetCategoryListForUser',
            protocol_dot_records__pb2.User.SerializeToString,
            protocol_dot_records__pb2.CategoryList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRecordById(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/records.Records/GetRecordById',
            protocol_dot_records__pb2.RecordRequest.SerializeToString,
            protocol_dot_records__pb2.Record.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetLastGroupForUser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/records.Records/GetLastGroupForUser',
            protocol_dot_records__pb2.User.SerializeToString,
            protocol_dot_records__pb2.RecordList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRecords(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/records.Records/GetRecords',
            protocol_dot_records__pb2.RecordQuery.SerializeToString,
            protocol_dot_records__pb2.RecordList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CountRecords(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/records.Records/CountRecords',
            protocol_dot_records__pb2.RecordQuery.SerializeToString,
            protocol_dot_records__pb2.RecordList.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from .grpc_client import RecordsClient
from .rest_client import RestApiClient


class AgentApiClient:
    def __init__(self, api_key, host="https://medsenger.ru", agent_id=None, debug=False, use_grpc=False, grpc_host=None):
        self.rest_client = RestApiClient(api_key, host, agent_id, debug)
        self.grpc_client = None
        self.user_cache = {}
        self.categories_cache = {}
        self.debug = debug

        if use_grpc:
            self.grpc_client = RecordsClient(host=grpc_host, debug=debug)

    def get_categories(self):
        if not self.grpc_client:
            return self.rest_client.get_categories()
        else:
            categories = self.grpc_client.get_categories()

            for category in categories:
                self.categories_cache[category['name']] = category

            return categories

    def get_available_categories(self, contract_id):
        if not self.grpc_client:
            return self.rest_client.get_available_categories(contract_id)
        else:
            if contract_id not in self.user_cache:
                self.get_patient_info(contract_id)

            if contract_id in self.user_cache:
                return self.grpc_client.get_categories_for_user(self.user_cache[contract_id])

            return []

    def get_patient_info(self, contract_id):
        result = self.rest_client.get_patient_info(contract_id)

        if self.debug:
            print(result)

        if result.get('id'):
            self.user_cache[contract_id] = result.get('id')

        return result

    def get_clinics_info(self):
        return self.rest_client.get_clinics_info()

    def get_records(self, contract_id, category_name=None, time_from=None, time_to=None, limit=None, offset=None,
                    group=False, return_count=False, inner_list=False):

        if not self.grpc_client:
            return self.rest_client.get_records(contract_id, category_name, time_from, time_to, limit, offset, group,
                                                return_count, inner_list)
        else:
            if contract_id not in self.user_cache:
                self.get_patient_info(contract_id)

            if return_count:
                method = self.grpc_client.count_records
            else:
                method = self.grpc_client.get_records

            return method(self.user_cache[contract_id], category_name, time_from, time_to, offset, limit, group, inner_list)

    def get_record_by_id(self, contract_id, record_id):
        if not self.grpc_client:
            return self.rest_client.get_record_by_id(contract_id, record_id)
        else:
            return self.grpc_client.get_record_by_id(record_id)

    def add_hooks(self, contract_id, names):
        return self.rest_client.add_hooks(contract_id, names)

    def remove_hooks(self, contract_id, names):
        return self.rest_client.remove_hooks(contract_id, names)

    def send_addition(self, contract_id, record_id, addition):
        return self.rest_client.send_addition(contract_id, record_id, addition)

    def add_record(self, contract_id, category_name, value, record_time=None, params=None, files=None, return_id=False):
        return self.rest_client.add_record(contract_id, category_name, value, record_time, params, files, return_id)

    def add_records(self, contract_id, values, record_time=None, params={}, return_id=False):
        return self.rest_client.add_records(contract_id, values, record_time, params, return_id)

    def send_message(self, contract_id, text, action_link=None, action_name=None, action_onetime=True,
                     only_doctor=False,
                     only_patient=False, action_deadline=None, is_urgent=False, need_answer=False,
                     attachments=None, action_big=True, send_from=None, forward_to_doctor=True, action_type='action'):
        return self.rest_client.send_message(contract_id, text, action_link, action_name, action_onetime,
                                             only_doctor,
                                             only_patient, action_deadline, is_urgent, need_answer,
                                             attachments, action_big, send_from, forward_to_doctor, action_type)

    def finish_task(self, contract_id, task_id):
        return self.rest_client.finish_task(contract_id, task_id)

    def delete_task(self, contract_id, task_id):
        return self.rest_client.delete_task(contract_id, task_id)

    def add_task(self, contract_id, text, target_number=1, date=None, important=False, action_link=None):
        return self.rest_client.add_task(contract_id, text, target_number, date, important, action_link)

    def request_payment(self, inv_id, amount, title):
        return self.rest_client.request_payment(inv_id, amount, title)

    def send_order(self, contract_id, order, receiver_id=None, params=None):
        return self.rest_client.send_order(contract_id, order, receiver_id, params)

    def get_agent_token(self, contract_id):
        return self.rest_client.get_agent_token(contract_id)

    def download_file(self, *args, **kwargs):
        return self.get_file(*args, **kwargs)

    def download_attachment(self, *args, **kwargs):
        return self.get_attachment(*args, **kwargs)

    def download_image(self, *args, **kwargs):
        return self.get_image(*args, **kwargs)

    def get_file(self, contract_id, file_id):
        return self.rest_client.get_file(contract_id, file_id)

    def get_attachment(self, attachment_id):
        return self.rest_client.get_attachment(attachment_id)

    def get_image(self, image_id, size):
        return self.rest_client.get_image(image_id, size)

    def update_cache(self, contract_id):
        return self.rest_client.update_cache(contract_id)

    def set_info_materials(self, contract_id, materials):
        return self.rest_client.set_info_materials(contract_id, materials)

    def ajax_url(self, action, contract_id, agent_token):
        # TODO fix
        return self.host.replace('8001',
                                 '8000') + "/api/client/agents/{agent_id}/?action={action}&contract_id={contract_id}&agent_token={agent_token}".format(
            agent_id=self.agent_id, action=action, contract_id=contract_id, agent_token=agent_token
        )





from medsenger_api import AgentApiClient, prepare_file
from unittest import TestCase
from config import *

class TestPayment(TestCase):

    def create_client(self):
        return AgentApiClient(API_KEY, MAIN_HOST, AGENT_ID, True)

    def test_request_payment(self):
        print(self.create_client().request_payment("test_1", 100, "Test payment"))





# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protocol/records.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16protocol/records.proto\x12\x07records\"\x07\n\x05\x45mpty\"\x12\n\x04User\x12\n\n\x02id\x18\x01 \x01(\x05\"\x1b\n\rRecordRequest\x12\n\n\x02id\x18\x01 \x01(\x03\"5\n\x0c\x43\x61tegoryList\x12%\n\ncategories\x18\x01 \x03(\x0b\x32\x11.records.Category\"\x96\x02\n\x08\x43\x61tegory\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x18\n\x0b\x64\x65scription\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04unit\x18\x04 \x01(\tH\x01\x88\x01\x01\x12\x11\n\x04type\x18\x05 \x01(\tH\x02\x88\x01\x01\x12#\n\x16\x64\x65\x66\x61ult_representation\x18\x06 \x01(\tH\x03\x88\x01\x01\x12\x16\n\tis_legacy\x18\x07 \x01(\x08H\x04\x88\x01\x01\x12\x18\n\x0bsubcategory\x18\x08 \x01(\tH\x05\x88\x01\x01\x42\x0e\n\x0c_descriptionB\x07\n\x05_unitB\x07\n\x05_typeB\x19\n\x17_default_representationB\x0c\n\n_is_legacyB\x0e\n\x0c_subcategory\"=\n\nRecordList\x12 \n\x07records\x18\x01 \x03(\x0b\x32\x0f.records.Record\x12\r\n\x05\x63ount\x18\x02 \x01(\x05\"4\n\x0cRecordSource\x12\x0f\n\x02id\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12\x0c\n\x04name\x18\x02 \x01(\tB\x05\n\x03_id\"4\n\nRecordFile\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\"\x87\x02\n\x06Record\x12\n\n\x02id\x18\x01 \x01(\x03\x12\r\n\x05value\x18\x02 \x01(\t\x12\x13\n\x0b\x63\x61tegory_id\x18\x03 \x01(\x03\x12\x0f\n\x07user_id\x18\x04 \x01(\x05\x12\x12\n\ncreated_at\x18\x05 \x01(\x03\x12\x12\n\nupdated_at\x18\x06 \x01(\x03\x12\x0e\n\x06params\x18\x07 \x01(\t\x12\x11\n\tadditions\x18\x08 \x01(\t\x12\r\n\x05group\x18\t \x01(\t\x12*\n\x06source\x18\n \x01(\x0b\x32\x15.records.RecordSourceH\x00\x88\x01\x01\x12+\n\x0e\x61ttached_files\x18\x0b \x03(\x0b\x32\x13.records.RecordFileB\t\n\x07_source\"\xe2\x01\n\x0bRecordQuery\x12\x14\n\x0c\x63\x61tegory_ids\x18\x01 \x03(\x03\x12\x1b\n\x0e\x66rom_timestamp\x18\x02 \x01(\x03H\x00\x88\x01\x01\x12\x19\n\x0cto_timestamp\x18\x03 \x01(\x03H\x01\x88\x01\x01\x12\x13\n\x06offset\x18\x04 \x01(\x03H\x02\x88\x01\x01\x12\x12\n\x05limit\x18\x05 \x01(\x03H\x03\x88\x01\x01\x12\x0f\n\x07user_id\x18\x06 \x01(\x05\x12\x12\n\nwith_group\x18\x07 \x01(\x08\x42\x11\n\x0f_from_timestampB\x0f\n\r_to_timestampB\t\n\x07_offsetB\x08\n\x06_limit2\xf8\x02\n\x07Records\x12:\n\x0fGetCategoryList\x12\x0e.records.Empty\x1a\x15.records.CategoryList\"\x00\x12@\n\x16GetCategoryListForUser\x12\r.records.User\x1a\x15.records.CategoryList\"\x00\x12:\n\rGetRecordById\x12\x16.records.RecordRequest\x1a\x0f.records.Record\"\x00\x12;\n\x13GetLastGroupForUser\x12\r.records.User\x1a\x13.records.RecordList\"\x00\x12\x39\n\nGetRecords\x12\x14.records.RecordQuery\x1a\x13.records.RecordList\"\x00\x12;\n\x0c\x43ountRecords\x12\x14.records.RecordQuery\x1a\x13.records.RecordList\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protocol.records_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _EMPTY._serialized_start=35
  _EMPTY._serialized_end=42
  _USER._serialized_start=44
  _USER._serialized_end=62
  _RECORDREQUEST._serialized_start=64
  _RECORDREQUEST._serialized_end=91
  _CATEGORYLIST._serialized_start=93
  _CATEGORYLIST._serialized_end=146
  _CATEGORY._serialized_start=149
  _CATEGORY._serialized_end=427
  _RECORDLIST._serialized_start=429
  _RECORDLIST._serialized_end=490
  _RECORDSOURCE._serialized_start=492
  _RECORDSOURCE._serialized_end=544
  _RECORDFILE._serialized_start=546
  _RECORDFILE._serialized_end=598
  _RECORD._serialized_start=601
  _RECORD._serialized_end=864
  _RECORDQUERY._serialized_start=867
  _RECORDQUERY._serialized_end=1093
  _RECORDS._serialized_start=1096
  _RECORDS._serialized_end=1472
# @@protoc_insertion_point(module_scope)



import json
import time
import grpc
from .utils import *
from .protocol import records_pb2_grpc as pb2_grpc
from .protocol import records_pb2 as pb2


class RecordsClient(object):
    def __init__(self, debug=False, host=None):
        if not host:
            host = "medsenger.ru"

        self.host = host
        self.server_port = 50051
        self.__categories_by_id = {}
        self.__categories_by_name = {}
        self.__debug = debug

        # instantiate a channel
        self.channel = grpc.insecure_channel(
            '{}:{}'.format(self.host, self.server_port), options=[
                ('grpc.max_send_message_length', 50 * 1024 * 1024),
                ('grpc.max_receive_message_length', 50 * 1024 * 1024),
                ('grpc.keepalive_time_ms', 10000),
                ('grpc.keepalive_timeout_ms', 5000),
                ('grpc.keepalive_permit_without_calls', True),
                ('grpc.http2.max_pings_without_data', 0),
                ('grpc.http2.min_time_between_pings_ms', 10000),
                ('grpc.http2.min_ping_interval_without_data_ms', 5000)
            ])

        # bind the client and the server
        self.stub = pb2_grpc.RecordsStub(self.channel)

        if self.__debug:
            print("Connected to GRPC")

    def __find_category_by_id(self, id):
        if not self.__categories_by_id or id not in self.__categories_by_id:
            print("Loading categories")
            self.get_categories()

        return self.__categories_by_id.get(id)

    def __find_category_by_name(self, name):
        if not self.__categories_by_name or name not in self.__categories_by_name:
            print("Loading categories")
            self.get_categories()

        return self.__categories_by_name.get(name)

    def __find_ids_for_categories(self, category_names):
        if not self.__categories_by_name:
            print("Loading categories")
            self.get_categories()

        return [self.__categories_by_name[name].id for name in category_names if name in self.__categories_by_name]

    def __present_category(self, category):
        return {
            "id": category.id,
            "name": category.name,
            "description": category.description,
            "unit": category.unit,
            "type": category.type,
            "default_representation": category.default_representation,
            "is_legacy": category.is_legacy,
            "subcategory": category.subcategory if category.subcategory else None
        }

    def __present_file(self, file):
        return {
            "id": file.id,
            "name": file.name,
            "type": file.type,
        }

    def __convert_value(self, value, category):
        if category.type == 'integer':
            return int(float(value))
        if category.type == 'float':
            return float(value)
        return value

    def __present_record(self, record, with_category=False):
        category = self.__find_category_by_id(record.category_id)

        presentation = {
            "id": record.id,
            "value": self.__convert_value(record.value, category),
            "timestamp": record.created_at,
            "uploaded": record.updated_at,
            "source": {
                "id": record.source.id if record.source.id else None,
                "name": record.source.name
            },
            "group": record.group if record.group else None,
            "params": json.loads(record.params),
            "additions": json.loads(record.additions),
            "attached_files": [self.__present_file(file) for file in record.attached_files],
        }

        if with_category:
            presentation['category_info'] = self.__present_category(category)

        if self.__debug:
            print(presentation)

        return presentation

    def __make_request(self, F, *args, timeout=10, tries=3):
        if self.__debug:
            print(gts(), " GRPC request to {} with params {}".format(F, args))

        exception = None
        for i in range(tries):
            try:
                result = F(*args, timeout=timeout)
                break
            except Exception as e:
                print("Exception in GRPC request: ", e)

                exception = e

                if self.__debug:
                    print("Retrying request... {}".format(i + 1))
        else:
            print("Request failed!")
            raise GrpcConnectionError("GRPC request error to {} with params {}: {}".format(F, args, exception))

        return result

    @safe
    def get_categories(self):
        request = pb2.Empty()

        result = self.__make_request(self.stub.GetCategoryList, request, timeout=10)

        for category in result.categories:
            self.__categories_by_id[category.id] = category
            self.__categories_by_name[category.name] = category

        return [self.__present_category(category) for category in result.categories]

    @safe
    def get_categories_for_user(self, user_id):
        request = pb2.User(id=user_id)

        result = self.__make_request(self.stub.GetCategoryListForUser, request, timeout=10)

        return [self.__present_category(category) for category in result.categories]

    @safe
    def get_record_by_id(self, record_id):
        request = pb2.RecordRequest(id=record_id)

        result = self.__make_request(self.stub.GetRecordById, request, timeout=10)

        return [self.__present_record(record) for record in result]

    def __aggregate_records(self, method, user_id, category_name, from_timestamp=0, to_timestamp=int(time.time()),
                            offset=0,
                            limit=None, group=False, inner_list=False):
        full_list = not category_name or ',' in category_name or inner_list

        category_names = category_name.split(',')
        category_ids = self.__find_ids_for_categories(category_names)

        if category_names and not category_ids:
            if self.__debug:
                print("Category search failed:", category_names, self.__categories_by_name.keys())

            return None, 0

        if from_timestamp is not None:
            from_timestamp = int(from_timestamp)

        if to_timestamp is not None:
            to_timestamp = int(to_timestamp)

        if offset is not None:
            offset = int(offset)

        if limit is not None:
            limit = int(limit)

        request = pb2.RecordQuery(user_id=user_id, category_ids=category_ids, from_timestamp=from_timestamp,
                                  to_timestamp=to_timestamp, offset=offset, limit=limit, with_group=group)

        result = self.__make_request(method, request, timeout=25, tries=2)
        records = None

        if full_list:
            if result.records:
                records = [self.__present_record(record, with_category=True) for record in result.records]
        else:
            if result.records:
                records = {
                    "category": self.__present_category(self.__categories_by_name.get(category_name)),
                    "values": [self.__present_record(record, with_category=False) for record in result.records]
                }

        return records, result.count

    @safe
    def get_records(self, user_id, category_name, from_timestamp=0, to_timestamp=int(time.time()), offset=0,
                    limit=None, group=False, inner_list=False):
        records, count = self.__aggregate_records(self.stub.GetRecords, user_id, category_name, from_timestamp,
                                                  to_timestamp,
                                                  offset, limit, group, inner_list)
        return records

    @safe
    def count_records(self, user_id, category_name, from_timestamp=0, to_timestamp=int(time.time()), offset=0,
                      limit=None, group=False, inner_list=False):

        records, count = self.__aggregate_records(self.stub.CountRecords, user_id, category_name, from_timestamp,
                                                  to_timestamp,
                                                  offset, limit, group, inner_list)

        return {"count": count}



import base64
import os
import sys
from datetime import datetime

class GrpcConnectionError(Exception):
    pass


def prepare_binary(name, data):
    import magic
    type = magic.from_buffer(data, mime=True)

    return {
        "name": name,
        "base64": base64.b64encode(data).decode('utf-8'),
        "type": type
    }


def prepare_file(filename):
    import magic
    import os

    type = magic.from_file(filename, mime=True)

    with open(filename, 'rb') as file:
        answer = {
            "name": filename.split(os.sep)[-1],
            "base64": base64.b64encode(file.read()).decode('utf-8'),
            "type": type
        }

    return answer

def gts():
    now = datetime.now()
    return now.strftime("%Y-%m-%d %H:%M:%S - ")

def safe(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except GrpcConnectionError as e:
            raise e
        except Exception as e:
            exc_type, exc_obj, exc_tb = sys.exc_info()
            file_name = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print(gts(), exc_type, file_name, exc_tb.tb_lineno, e, "CRITICAL")
            return None

    wrapper.__name__ = func.__name__
    return wrapper